package main

import (
	"bufio"
	"flag"
	"fmt"
	"net/url"
	"os"
	"sort"
	"strings"

	"github.com/edoardottt/golazy"
)

func main() {
	helpPtr := flag.Bool("h", false, "Show usage.")

	flag.Parse()

	if *helpPtr {
		help()
	}

	set := make(map[string]int)

	input := ScanTargets()
	for _, elem := range input {
		host := GetHost(elem)
		if host != "" {
			_, exists := set[host]
			if exists {
				set[host] += 1
			} else {
				set[host] = 1
			}
		}
	}

	// sort reverse.
	var (
		n = map[int][]string{}
		a = []int{}
	)

	for k, v := range set {
		n[v] = append(n[v], k)
	}

	for k := range n {
		a = append(a, k)
	}

	sort.Sort(sort.Reverse(sort.IntSlice(a)))

	for _, k := range a {
		for _, s := range n[k] {
			fmt.Printf("[ %d ] %s\n", k, s)
		}
	}
}

// help shows the usage.
func help() {
	var usage = `Take as input on stdin a list of urls and print on stdout all the hosts sorted.
	$> cat urls | eah`

	fmt.Println()
	fmt.Println(usage)
	fmt.Println()
	os.Exit(0)
}

// ScanTargets return the array of elements
// taken as input on stdin.
func ScanTargets() []string {
	var result []string

	// accept domains on stdin.
	sc := bufio.NewScanner(os.Stdin)
	for sc.Scan() {
		domain := strings.ToLower(sc.Text())
		result = append(result, domain)
	}

	return golazy.RemoveDuplicateValues(result)
}

// GetHost.
func GetHost(input string) string {
	u, err := url.Parse(input)
	if err != nil {
		return ""
	}

	return u.Host
}
